# -*- coding: UTF-8 -*-
from setuphelpers import *
import requests
import json

dist_dir = "distribution"
dist_path = makepath(programfiles, "Mozilla Firefox", dist_dir)
ext_dir = "extensions"
ext_path = makepath(dist_path, ext_dir)
policies_file = "policies.json"
policies_path = makepath(dist_path, policies_file)
list_pre_installed_lang = ["fr", "en-GB", "es-ES", "de", "it"]  # Reminder: 'en-US' will be native


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name
    arch_dict = {"x64": "win64", "x86": "win", "all": "win"}
    lang = control.locale
    policies_content = json_load_file(policies_file)

    # Getting latest version from official sources
    download_url = requests.head(
        "https://download.mozilla.org/?product=firefox-latest-ssl&os=%s&lang=en-US" % (arch_dict[control.architecture]), proxies=proxies
    ).headers["Location"]
    latest_bin = download_url.rsplit("/", 1)[1].replace("%20", "_")
    version = download_url.split("/")[-4]

    # Downloading latest binaries
    print("Latest %s version is: %s" % (app_name, version))
    print("Download URL is: %s" % download_url)
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print("Binary is present: %s" % latest_bin)

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Download Firefox Language Packs
    # https://addons.mozilla.org/firefox/addon/fran%C3%A7ais-language-pack/versions/
    # https://addons.mozilla.org/firefox/addon/english-gb-language-pack/versions/
    url_api_lp = "https://services.addons.mozilla.org/api/v3/addons/language-tools/?app=firefox&type=language"
    print("API used for language packs is: %s" % url_api_lp)
    version_major = control.version.split("-")[0].split(".")[0]

    if isdir(ext_dir):
        remove_tree(ext_dir)

    for lang in list_pre_installed_lang:
        json_read = json.loads(wgets(url_api_lp, proxies=proxies))
        for lp in json_read["results"]:
            if lp["target_locale"] == lang:
                url_lp = lp["url"] + "versions/"
                break

        # https://addons.mozilla.org/firefox/downloads/file/3605597/francais_language_pack-78.0buildid20200708170202-fx.xpi?src=dp-btn-primary
        # https://addons.mozilla.org/firefox/downloads/file/3605535/english_gb_language_pack-78.0buildid20200708170202-fx.xpi?src=dp-btn-primary

        for download_lp in bs_find_all(
            url_lp,
            "a",
            "class",
            "Button Button--action AMInstallButton-button Button--puffy",
            proxies=proxies,
            headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:%s.0) Gecko/20100101 Firefox/%s.0" % (version_major, version_major)},
        ):
            if version_major in download_lp["href"]:
                url_dl_lp = download_lp["href"]
                break

        # Forming polices.json file
        lp_name = "%s_language_pack.xpi" % lang
        ext_settings_content = {
            "langpack-%s@firefox.mozilla.org"
            % lang: {"install_url": "file://%s\\%s_language_pack.xpi" % (ext_path, lang), "installation_mode": "force_installed"}
        }
        policies_content["policies"]["ExtensionSettings"].update(ext_settings_content)

        print("Download url for language pack in %s is: %s" % (lang, url_dl_lp))
        print("Downloading: " + lp_name)
        wget(url_dl_lp, makepath(ext_dir, lp_name), proxies=proxies)

    # Downloading dictionaries
    # https://addons.mozilla.org/firefox/addon/dictionnaire-fran%C3%A7ais1/
    # https://addons.mozilla.org/firefox/addon/british-english-dictionary-2/
    url_api_dict_lang = "https://services.addons.mozilla.org/api/v3/addons/language-tools/?app=firefox&type=dictionary"
    print("API used for language dictionaries is: %s" % url_api_dict_lang)

    for lang in list_pre_installed_lang:
        # Debuggin DE lang
        if lang == "de":
            lang = "de-DE"
        json_read = json.loads(wgets(url_api_dict_lang, proxies=proxies))
        for dict_lang in json_read["results"]:
            if dict_lang["target_locale"] == lang:
                url_dl_dict_lang = "https://addons.mozilla.org/firefox/downloads/latest/%s/latest.xpi" % dict_lang["slug"]
                ext_add_on_id = dict_lang["guid"]
                break

        # https://addons.mozilla.org/firefox/downloads/latest/dictionnaire-fran%C3%A7ais1/latest.xpi
        # https://addons.mozilla.org/firefox/downloads/latest/british-english-dictionary-2/latest.xpi

        dict_lang_name = "%s_dictionary.xpi" % lang

        print("Download url for dictionary in %s is: %s" % (lang, url_dl_dict_lang))
        print("Downloading: " + dict_lang_name)
        wget(url_dl_dict_lang, makepath(ext_dir, dict_lang_name), proxies=proxies)

        ext_settings_dictionaries_content = {
            "%s" % ext_add_on_id: {"install_url": "file://%s\\%s" % (ext_path, dict_lang_name), "installation_mode": "force_installed"}
        }
        policies_content["policies"]["ExtensionSettings"].update(ext_settings_dictionaries_content)

    print("Creating %s file for language packs, dictionaries, extensions and configuration" % policies_file)
    if isfile(policies_file):
        remove_file(policies_file)
    json_write_file(policies_file, policies_content, indent=2)

    # Downloading extentions
    # uBlock Origin
    wget("https://addons.mozilla.org/firefox/downloads/latest/ublock-origin/latest.xpi", makepath(ext_dir, "ublock-origin.xpi"), proxies=proxies)

    # Validating or not update-package-sources
    return package_updated
